/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import de.chaosdorf.mete.UserId
import de.chaosdorf.meteroid.model.Server
import de.chaosdorf.meteroid.model.ServerId
import de.chaosdorf.meteroid.model.ServerRepository
import de.chaosdorf.meteroid.storage.AccountPreferences
import de.chaosdorf.meteroid.sync.AccountProvider
import de.chaosdorf.meteroid.sync.DrinkSyncHandler
import de.chaosdorf.meteroid.sync.PurchaseSyncHandler
import de.chaosdorf.meteroid.sync.UserSyncHandler
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.distinctUntilChanged
import kotlinx.coroutines.flow.flatMapLatest
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.mapLatest
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.flow.stateIn
import javax.inject.Inject

@HiltViewModel
class AppViewModel @Inject constructor(
  accountProvider: AccountProvider,
  private val accountPreferences: AccountPreferences,
  private val serverRepository: ServerRepository,
  private val userSyncHandler: UserSyncHandler,
  private val drinkSyncHandler: DrinkSyncHandler,
  private val purchaseSyncHandler: PurchaseSyncHandler
) : ViewModel() {
  val initState: StateFlow<InitState> = accountPreferences.state
    .flatMapLatest { preferences ->
      serverRepository.getAllFlow()
        .mapLatest { it.map(Server::serverId) }
        .mapLatest { servers ->
          if (servers.isEmpty()) InitState.CREATE_SERVER
          else if (!servers.contains(preferences.server)) InitState.SELECT_SERVER
          else if (preferences.user == null) InitState.SELECT_USER
          else InitState.HOME
        }
    }.stateIn(viewModelScope, SharingStarted.WhileSubscribed(), InitState.LOADING)

  init {
    accountProvider.account.distinctUntilChanged().onEach { account ->
      account?.let { (server, user) ->
        userSyncHandler.sync(server)
        drinkSyncHandler.sync(server)
        user?.let { user ->
          purchaseSyncHandler.sync(Pair(server, user.userId))
        }
      }
    }.launchIn(viewModelScope)
  }

  suspend fun selectServer(server: ServerId) {
    accountPreferences.setServer(server)
    accountPreferences.setUser(null)
  }

  suspend fun selectUser(user: UserId) {
    accountPreferences.setUser(user)
  }

  enum class InitState {
    LOADING,
    CREATE_SERVER,
    SELECT_SERVER,
    SELECT_USER,
    HOME
  }
}
