/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui.purchases

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import de.chaosdorf.meteroid.model.Drink
import de.chaosdorf.meteroid.model.DrinkRepository
import de.chaosdorf.meteroid.model.Purchase
import de.chaosdorf.meteroid.model.PurchaseRepository
import de.chaosdorf.meteroid.model.Server
import de.chaosdorf.meteroid.model.User
import de.chaosdorf.meteroid.sync.AccountProvider
import de.chaosdorf.meteroid.sync.PurchaseSyncHandler
import de.chaosdorf.meteroid.sync.SyncHandler
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.flatMapLatest
import kotlinx.coroutines.flow.flowOf
import kotlinx.coroutines.flow.mapLatest
import kotlinx.coroutines.flow.stateIn
import javax.inject.Inject
import kotlin.time.Duration.Companion.minutes

@HiltViewModel
class PurchaseViewModel @Inject constructor(
  accountProvider: AccountProvider,
  repository: PurchaseRepository,
  drinkRepository: DrinkRepository,
  syncHandler: PurchaseSyncHandler
) : ViewModel() {
  val account: StateFlow<Pair<Server, User?>?> = accountProvider.account
    .stateIn(viewModelScope, SharingStarted.WhileSubscribed(), null)

  val purchases: StateFlow<List<Pair<Purchase, Drink?>>> = accountProvider.account
    .flatMapLatest { account ->
      account?.let { (server, user) ->
        user?.let { user ->
          combine(
            repository.getAllFlow(server.serverId, user.userId),
            drinkRepository.getAllFlow(server.serverId)
          ) { purchases, drinks ->
            purchases.map { purchase ->
              Pair(purchase, drinks.firstOrNull { drink -> drink.drinkId == purchase.drinkId })
            }
          }
        }
      } ?: flowOf(emptyList())
    }.mapLatest { list ->
      list.mergeAdjecentDeposits()
        .filter { it.second != null || it.first.difference != 0.0 }
    }.stateIn(viewModelScope, SharingStarted.WhileSubscribed(), emptyList())

  val syncState: StateFlow<SyncHandler.State> = syncHandler.state
}

fun List<Pair<Purchase, Drink?>>.mergeAdjecentDeposits(): List<Pair<Purchase, Drink?>> {
  val result = mutableListOf<Pair<Purchase, Drink?>>()
  for (entry in this) {
    val previous = result.lastOrNull()
    if (previous != null
      && previous.first.difference > 0
      && entry.first.difference > 0
      && previous.second == null
      && entry.second == null
      && entry.first.createdAt.minus(previous.first.createdAt) < 5.minutes
    ) {
      result.removeLast()
      result.add(
        Pair(
          entry.first.copy(difference = entry.first.difference + previous.first.difference),
          null
        )
      )
    } else {
      result.add(entry)
    }
  }
  return result
}
